@@ -1,14 +1,16 @@
 import random
 from pathlib import Path
-from typing import Union, List
+from typing import Union, List, Optional
+from torch import Tensor
 
 import torch
 from torch.nn.utils.rnn import pad_sequence
 
 from ..core.transforms_interface import BaseWaveformTransform, EmptyPathException
 from ..utils.convolution import convolve
-from ..utils.file import find_audio_files
+from ..utils.file import find_audio_files_in_paths
 from ..utils.io import Audio
+from ..utils.object_dict import ObjectDict
 
 
 class ApplyImpulseResponse(BaseWaveformTransform):
@@ -16,11 +18,16 @@ class ApplyImpulseResponse(BaseWaveformTransform):
     Convolve the given audio with impulse responses.
     """
 
+    supported_modes = {"per_batch", "per_example", "per_channel"}
+
     # Note: This transform has only partial support for multichannel audio. IRs that are not
     # mono get mixed down to mono before they are convolved with all channels in the input.
     supports_multichannel = True
     requires_sample_rate = True
 
+    supports_target = False  # FIXME: some work is needed to support targets (see FIXMEs in apply_transform)
+    requires_target = False
+
     def __init__(
         self,
         ir_paths: Union[List[Path], List[str], Path, str],
@@ -30,6 +37,8 @@ class ApplyImpulseResponse(BaseWaveformTransform):
         p: float = 0.5,
         p_mode: str = None,
         sample_rate: int = None,
+        target_rate: int = None,
+        output_type: Optional[str] = None,
     ):
         """
         :param ir_paths: Either a path to a folder with audio files or a list of paths to audio files.
@@ -43,14 +52,20 @@ class ApplyImpulseResponse(BaseWaveformTransform):
         :param p:
         :param p_mode:
         :param sample_rate:
+        :param target_rate:
         """
-        super().__init__(mode, p, p_mode, sample_rate)
 
-        if isinstance(ir_paths, (list, tuple, set)):
-            # TODO: check that one can read audio files
-            self.ir_paths = list(ir_paths)
-        else:
-            self.ir_paths = find_audio_files(ir_paths)
+        super().__init__(
+            mode=mode,
+            p=p,
+            p_mode=p_mode,
+            sample_rate=sample_rate,
+            target_rate=target_rate,
+            output_type=output_type,
+        )
+
+        # TODO: check that one can read audio files
+        self.ir_paths = find_audio_files_in_paths(ir_paths)
 
         if sample_rate is not None:
             self.audio = Audio(sample_rate=sample_rate, mono=True)
@@ -61,9 +76,15 @@ class ApplyImpulseResponse(BaseWaveformTransform):
         self.convolve_mode = convolve_mode
         self.compensate_for_propagation_delay = compensate_for_propagation_delay
 
-    def randomize_parameters(self, selected_samples, sample_rate: int = None):
+    def randomize_parameters(
+        self,
+        samples: Tensor = None,
+        sample_rate: Optional[int] = None,
+        targets: Optional[Tensor] = None,
+        target_rate: Optional[int] = None,
+    ):
 
-        batch_size, _, _ = selected_samples.shape
+        batch_size, _, _ = samples.shape
 
         audio = self.audio if hasattr(self, "audio") else Audio(sample_rate, mono=True)
 
@@ -77,20 +98,25 @@ class ApplyImpulseResponse(BaseWaveformTransform):
 
         self.transform_parameters["ir_paths"] = random_ir_paths
 
-    def apply_transform(self, selected_samples, sample_rate: int = None):
+    def apply_transform(
+        self,
+        samples: Tensor = None,
+        sample_rate: Optional[int] = None,
+        targets: Optional[Tensor] = None,
+        target_rate: Optional[int] = None,
+    ) -> ObjectDict:
 
-        batch_size, num_channels, num_samples = selected_samples.shape
+        batch_size, num_channels, num_samples = samples.shape
 
         # (batch_size, 1, max_ir_length)
-        ir = self.transform_parameters["ir"].to(selected_samples.device)
+        ir = self.transform_parameters["ir"].to(samples.device)
 
         convolved_samples = convolve(
-            selected_samples, ir.expand(-1, num_channels, -1), mode=self.convolve_mode
+            samples, ir.expand(-1, num_channels, -1), mode=self.convolve_mode
         )
 
         if self.compensate_for_propagation_delay:
             propagation_delays = ir.abs().argmax(dim=2, keepdim=False)[:, 0]
-
             convolved_samples = torch.stack(
                 [
                     convolved_sample[
@@ -103,7 +129,17 @@ class ApplyImpulseResponse(BaseWaveformTransform):
                 dim=0,
             )
 
-            return convolved_samples
+            return ObjectDict(
+                samples=convolved_samples,
+                sample_rate=sample_rate,
+                targets=targets,  # FIXME compensate targets as well?
+                target_rate=target_rate,
+            )
 
         else:
-            return convolved_samples[..., :num_samples]
+            return ObjectDict(
+                samples=convolved_samples[..., :num_samples],
+                sample_rate=sample_rate,
+                targets=targets,  # FIXME crop targets as well?
+                target_rate=target_rate,
+            )
