@@ -1,7 +1,10 @@
 import torch
 import typing
+from typing import Optional
+from torch import Tensor
 
 from ..core.transforms_interface import BaseWaveformTransform
+from ..utils.object_dict import ObjectDict
 
 
 class PeakNormalization(BaseWaveformTransform):
@@ -16,8 +19,14 @@ class PeakNormalization(BaseWaveformTransform):
     untouched.
     """
 
+    supported_modes = {"per_batch", "per_example", "per_channel"}
+
+    supports_multichannel = True
     requires_sample_rate = False
 
+    supports_target = True
+    requires_target = False
+
     def __init__(
         self,
         apply_to="all",
@@ -25,16 +34,29 @@ class PeakNormalization(BaseWaveformTransform):
         p: float = 0.5,
         p_mode: typing.Optional[str] = None,
         sample_rate: typing.Optional[int] = None,
+        target_rate: typing.Optional[int] = None,
+        output_type: Optional[str] = None,
     ):
-        super().__init__(mode, p, p_mode, sample_rate)
+        super().__init__(
+            mode=mode,
+            p=p,
+            p_mode=p_mode,
+            sample_rate=sample_rate,
+            target_rate=target_rate,
+            output_type=output_type,
+        )
         assert apply_to in ("all", "only_too_loud_sounds")
         self.apply_to = apply_to
 
     def randomize_parameters(
-        self, selected_samples, sample_rate: typing.Optional[int] = None
+        self,
+        samples: Tensor = None,
+        sample_rate: Optional[int] = None,
+        targets: Optional[Tensor] = None,
+        target_rate: Optional[int] = None,
     ):
         # Compute the most extreme value of each multichannel audio snippet in the batch
-        most_extreme_values, _ = torch.max(torch.abs(selected_samples), dim=-1)
+        most_extreme_values, _ = torch.max(torch.abs(samples), dim=-1)
         most_extreme_values, _ = torch.max(most_extreme_values, dim=-1)
 
         if self.apply_to == "all":
@@ -55,9 +77,22 @@ class PeakNormalization(BaseWaveformTransform):
                 most_extreme_values[self.transform_parameters["selector"]], (-1, 1, 1)
             )
 
-    def apply_transform(self, selected_samples, sample_rate: typing.Optional[int] = None):
+    def apply_transform(
+        self,
+        samples: Tensor = None,
+        sample_rate: Optional[int] = None,
+        targets: Optional[Tensor] = None,
+        target_rate: Optional[int] = None,
+    ) -> ObjectDict:
+
         if "divisors" in self.transform_parameters:
-            selected_samples[
-                self.transform_parameters["selector"]
-            ] /= self.transform_parameters["divisors"]
-        return selected_samples
+            samples[self.transform_parameters["selector"]] /= self.transform_parameters[
+                "divisors"
+            ]
+
+        return ObjectDict(
+            samples=samples,
+            sample_rate=sample_rate,
+            targets=targets,
+            target_rate=target_rate,
+        )
